#############
# constants #
#############

# fields
from dada_settings.constants.field_defaults import *


# files
from dada_settings.constants.file_defaults import *
from dada_settings.constants.file_bundle_defaults import *
from dada_settings.constants.file_code_defaults import *
from dada_settings.constants.file_audio_defaults import *
from dada_settings.constants.file_data_defaults import *
from dada_settings.constants.file_doc_defaults import *
from dada_settings.constants.file_model_defaults import *
from dada_settings.constants.file_image_defaults import *
from dada_settings.constants.file_video_defaults import *
from dada_settings.constants.file_store_defaults import *

# etc
from dada_settings.constants.tag_defaults import *
from dada_settings.constants.theme_defaults import *

##################
# model defaults #
##################

# files
from dada_settings.models.file_bundle_defaults import *
from dada_settings.models.file_defaults import *
from dada_settings.models.file_code_defaults import *
from dada_settings.models.file_data_defaults import *
from dada_settings.models.file_doc_defaults import *
from dada_settings.models.file_audio_defaults import *
from dada_settings.models.file_image_defaults import *
from dada_settings.models.file_video_defaults import *
from dada_settings.models.file_model_defaults import *

# etc
from dada_settings.models.user_defaults import *
from dada_settings.models.tag_defaults import *
from dada_settings.models.desktop_defaults import *
from dada_settings.models.folder_defaults import *

#############################
# field generation defaults #
#############################

# gen defaults
from dada_settings.gen.desktop_defaults import *
from dada_settings.gen.field_defaults import *
from dada_settings.gen.folder_defaults import *


#####################################
# File extension / mimetype lookups #
#####################################


FILE_VALID_TYPE_EXT_MIMETYPE = {
    "audio": AUDIO_DEFAULTS_VALID_EXT_MIMETYPE,
    "video": VIDEO_DEFAULTS_VALID_EXT_MIMETYPE,
    "image": IMAGE_DEFAULTS_VALID_EXT_MIMETYPE,
    "model": MODEL_DEFAULTS_VALID_EXT_MIMETYPE,
    "bundle": BUNDLE_DEFAULTS_VALID_EXT_MIMETYPE,
    "data": DATA_DEFAULTS_VALID_EXT_MIMETYPE,
    "code": CODE_DEFAULTS_VALID_EXT_MIMETYPE,
    "doc": DOC_DEFAULTS_VALID_EXT_MIMETYPE,
    "raw": {},
}


def _rev(d):
    return {v: k for k, v in d.items()}


FILE_VALID_TYPE_MIMETYPE_EXT = {
    "audio": _rev(AUDIO_DEFAULTS_VALID_EXT_MIMETYPE),
    "video": _rev(VIDEO_DEFAULTS_VALID_EXT_MIMETYPE),
    "image": _rev(IMAGE_DEFAULTS_VALID_EXT_MIMETYPE),
    "model": _rev(MODEL_DEFAULTS_VALID_EXT_MIMETYPE),
    "bundle": _rev(BUNDLE_DEFAULTS_VALID_EXT_MIMETYPE),
    "data": _rev(DATA_DEFAULTS_VALID_EXT_MIMETYPE),
    "code": _rev(CODE_DEFAULTS_VALID_EXT_MIMETYPE),
    "doc": _rev(DOC_DEFAULTS_VALID_EXT_MIMETYPE),
    "raw": _rev({}),
}

FILE_VALID_EXT_MIMETYPE = {
    kk: vv for k, v in FILE_VALID_TYPE_EXT_MIMETYPE.items() for kk, vv in v.items()
}

FILE_VALID_MIMETYPE_EXT = {
    kk: vv for k, v in FILE_VALID_TYPE_MIMETYPE_EXT.items() for kk, vv in v.items()
}

# ###########################################
# Default Fields by entity type / file type #
# ###########################################

FIELD_TYPE_DEFAULTS = {
    "file": {
        "audio": AUDIO_DEFAULTS_DEFAULT_FIELDS,
        "bundle": BUNDLE_DEFAULTS_DEFAULT_FIELDS,
        "code": CODE_DEFAULTS_DEFAULT_FIELDS,
        "doc": DOC_DEFAULTS_DEFAULT_FIELDS,
        "data": DATA_DEFAULTS_DEFAULT_FIELDS,
        "image": IMAGE_DEFAULTS_DEFAULT_FIELDS,
        "model": MODEL_DEFAULTS_DEFAULT_FIELDS,
        "video": VIDEO_DEFAULTS_DEFAULT_FIELDS,
        "raw": [],
    },
    "desktop": DESKTOP_DEFAULTS_DEFAULT_FIELDS,
    "folder": FOLDER_DEFAULTS_DEFAULT_FIELDS,
    "tag": TAG_DEFAULTS_DEFAULT_FIELDS,
    "user": USER_DEFAULTS_DEFAULT_FIELDS,
}

# construct list of default fields
FIELD_DEFAULTS = []
for k, v in FIELD_TYPE_DEFAULTS.items():
    if isinstance(v, dict):
        for kk, vv in v.items():
            FIELD_DEFAULTS.extend(vv)
    else:
        FIELD_DEFAULTS.extend(v)

FIELD_DEFAULT_NAMES = [v["name"] for v in FIELD_DEFAULTS if "name" in v]



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT MODEL FIELDS
# ////////////////

MODEL_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["model"],
}

MODEL_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "num_layers",
        "type": "int",
        "info": "The number of layers in this model",
    },
]

MODEL_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "model", MODEL_DEFAULTS_DEFAULT_FIELDS_INIT, MODEL_DEFAULTS_DEFAULT_FIELD_PROPS
)



import os

DEBUG = False
SQLALCHEMY_DATABASE_URI = os.getenv(
    "DADA_SQLALCHEMY_DATABASE_URI_DOCKER",
    "postgresql://postgres:secret@postgres:5432/dada",
)
CELERY_BROKER_URL = os.getenv("DADA_CELERY_BROKER_URL_DOCKER", "redis://redis:6379")
CELERY_RESULT_BACKEND = os.getenv(
    "DADA_CELERY_RESULT_BACKEND_DOCKER", "redis://redis:6379"
)

# redis cache
REDIS_CACHE_URL = os.getenv("DADA_REDIS_CACHE_URL", "redis://redis:6379/")
REDIS_CACHE_DB = os.getenv("DADA_REDIS_CACHE_DB", "1")



# ///////////////////////////////////////
# Random Desktop Generation
# ///////////////////////////////////////


DESKTOP_DEFAULTS_RANDOM_DESKTOP_NAME_WORD_1 = [
    "dev",
    "prod",
    "test",
    "bar",
    "baz",
    "uqbar",
    "jazz",
    "funk",
    "rhythm",
    "ambient",
]
DESKTOP_DEFAULTS_RANDOM_DESKTOP_NAME_WORD_2 = [
    "space",
    "zone",
    "collective",
    "habitat",
    "forest",
    "meadow",
    "function",
    "anti-platform",
    "environment",
]



# /////////////////
# FILE EXTENSIONS
# /////////////////

VIDEO_DEFAULTS_VALID_EXT_MIMETYPE = {
    "flv": "video/x-flv",
    "mp4": "video/mp4",
    "m3u8": "application/x-mpegurl",
    "ts": "video/mp2t",
    "3gp": "video/3gpp",
    "mov": "video/quicktime",
    "avi": "video/x-msvideo",
    "wmv": "video/x-ms-wmv",
    "mpeg": "video/mpeg",
}

# metadata extraction

VIDEO_DEFAULTS_DEFAULT_METADATA_EXTRACTION = "ffp"



import os

DEBUG = True
SQLALCHEMY_DATABASE_URI = os.getenv(
    "DADA_SQLALCHEMY_DATABASE_URI_DEV", "postgresql://localhost:5432/dada_dev"
)
SQLALCHEMY_ECHO = os.getenv("DADA_SQLALCHEMY_ECHO", "false") == "true"
CELERY_ALWAYS_EAGER = os.getenv("DADA_CELERY_ALWAYS_EAGER", "true") == "true"
S3_BUCKET = os.getenv("DADA_S3_BUCKET_DEV", "dev.dada.globally.ltd")



from dada_settings.utils import join_fields

# ///////////////////////////////////////
# Default Desktop Generation
# ///////////////////////////////////////

# DEFAULT DESKTOPS
DESKTOP_DEFAULTS = [
    {
        "name": "dev",
        "info": "Your default, private environment",
        "is_private": True,
    },
    {
        "name": "shared",
        "info": "Your public environment",
        "is_private": False,
    },
]

DESKTOP_DEFAULTS_DEFAULT_DESKTOP_NAMES = [f["name"] for f in DESKTOP_DEFAULTS]
DESKTOP_DEFAULTS_NUMBER_DEFAULT_DESKTOPS = len(DESKTOP_DEFAULTS_DEFAULT_DESKTOP_NAMES)


DESKTOP_DEFAULTS_DEFAULT_FIELDS = []



# /////////////////
# DATA FILE EXTENSIONS
# /////////////////

DATA_DEFAULTS_VALID_EXT_MIMETYPE = {
    "csv": "text/csv",
    "parquet": "text/x-parquet",
    "json": "application/json",
    "jsonl": "application/json+ld",
    "jsonld": "application/json+ld",
    "xml": "application/xml",
    "yaml": "application/x-yaml",
    "yml": "application/x-yaml",
}

# TODO: IMPLEMENT THIS LATER



BUNDLE_DEFAULTS_VALID_EXT_MIMETYPE = {
    "tar": "application/x-tar",
    "Z": "application/zip",
    "zip": "application/zip",
    "gzip": "application/gzip",
    "gz": "application/gzip",
    "7z": "application/x-7z-compressed",
    "rar": "application/vnd.rar",
    "bz": "application/x-bzip",
    "bz2": "application/x-bzip2",
}
BUNDLE_DEFAULTS_VALID_EXT = list(BUNDLE_DEFAULTS_VALID_EXT_MIMETYPE.keys())


# //
# Heuristics for detecting whether a bundle
# is of a particular subtype
# //
BUNDLE_DEFAULTS_GUESS_SUBTYPE_FROM_EXTENSTIONS = {
    "gltf": {
        "ext_must": ["gltf"],
        "ext_possible": ["gltf", "png", "jpg", "bin"],
        "dir_possible": ["textures"],
        "min_files": 1,
        "max_files": 20,
        "max_dir_depth": 2,
    },
    "ableton_project": {
        "pkg_endswith": ["Project"],
        "ext_must": ["als"],
        "ext_possible": ["wav", "mp3", "aif", "aiff", "m4a", "flac", "mp4", "mov"],
        "dir_possible": ["Samples", "Backup", "Ableton Project Info"],
        "min_files": 1,
    },
    "logic_project": {
        "pkg_endswith": ["logicx"],
        "ext_possible": [
            "logicx",
            "wav",
            "mp3",
            "aif",
            "aiff",
            "m4a",
            "flac",
            "mp4",
            "mov",
        ],
        "dir_possible": ["alternatives", "contents", "media", "resources"],
        "min_files": 1,
    },
    "git_repo": {
        "ext_possible": [
            "md",
            "py",
            "js",
            "yml",
            "R",
            "css",
            "html",
        ],
        "dir_possible": [".git"],
        "min_files": 1,
    },
    "audio_album": {
        "ext_must": ["als"],
        "ext_possible": [
            "wav",
            "mp3",
            "aif",
            "aiff",
            "bin",
            "m4a",
            "flac",
            "mp4",
            "mov",
        ],
        "dir_possible": [
            "mp3",
            "wav",
            "aiff",
            "aif",
            "m4a",
            "flac",
        ],  # eg album/mp3/file1.mp3
        "path_possible": ["cover.png", "cover.jpg", "cover.jpeg"],
        "min_files": 1,
        "max_dir_depth": 2,
    },
    "rekordbox_library": {
        "ext_must": ["xml"],
        "ext_possible": ["wav", "mp3", "aif", "m4a", "flac", "aiff"],
        "min_files": 1,
        "max_dir_depth": 2,
    },
    "itunes_library": {
        "ext_must": ["xml"],
        "ext_possible": [
            "wav",
            "mp3",
            "aif",
            "m4a",
            "flac",
            "aiff",
            "jpg",
            "png",
            "jpeg",
            "mov",
            "mp4",
        ],
        "dir_possible": ["music"],
        "path_possible": ["library.xml"],
        "min_files": 1,
        "max_dir_depth": 2,
    },
}

BUNDLE_DEFAULTS_SUBTYPES = [
    "gltf",
    "ableton_project",
    "logic_project",
    "git_repo",
    "audio_album",
    "rekordbox_library",
    "itunes_library",
    # TODO: MORE!
]



from typing import NewType, List, Dict, Any


FieldsList = NewType("FieldsList", List[Dict[str, Any]])


def join_fields(prefix: str, fields: FieldsList, props: Dict[str, Any]) -> FieldsList:
    """
    Join and prefix a list of fields with default properties
    :param prefix: The prefix to prepend to the keys of the fields
    :param fields: A dictionary of key/value fields
    :param props: A dictionary of props to join by default
    :return dict
    """
    out_fields = []
    for f in fields:
        combined = dict(list(f.items()) + list(props.items()))
        name = combined.get("name")
        if not name:
            raise ValueError(
                f"Could not construct {prefix} field because it was missing a name: {combined}"
            )

        # prefix the name
        combined["name"] = f"{prefix}_{name}"
        out_fields.append(combined)

    return out_fields



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT IMAGE FIELDS
# /////////////////

IMAGE_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["image"],
}

IMAGE_DEFAULTS_DEFAULT_FIELDS_INIT = [
    # /////////////////
    # EXIF DATA
    # ////////////////
    {"name": "artist", "type": "text", "is_searchable": True},
    {"name": "info", "type": "text", "is_searchable": True},
    {"name": "date_time", "type": "date"},
    {"name": "camera_make", "type": "text"},
    {"name": "camera_model", "type": "text"},
    {"name": "image_width", "type": "int"},
    {"name": "image_height", "type": "int"},
    {"name": "x_resolution", "type": "int_array"},
    {"name": "y_resolution", "type": "int_array"},
    {"name": "camera_owner_name", "type": "text_256", "is_searchable": True},
    {"name": "flash", "type": "smallint"},
    {"name": "color_space", "type": "smallint"},
    {"name": "exposure_time", "type": "int_array"},
    # # photo settings
    # { 'name': 'shutter_speed_value', 'type': 'int_array'},
    # { 'name': 'aperture_value', 'type': 'int_array'},
    # { 'name': 'exposure_bias_value', 'type': 'int_array'},
    # { 'name': 'max_aperture_value', 'type': 'int_array'},
    # { 'name': 'metering_mode', "type": "smallint"},
    # { 'name': 'focal_length', 'type': 'int_array'},
    # { 'name': 'compressed_bits_per_pixel', 'type': 'int_array'},
    # # { 'name': 'date_time_original', 'type': 'date'},
    # # { 'name': 'date_time_digitized', 'type': 'date'},
    # { 'name': 'color_space', "type": "smallint"},
    # { 'name': 'focal_plane_x_resolution', 'type': 'int_array'},
    # { 'name': 'subsec_time', "type": "smallint"},
    # # { 'name': 'subsec_time_original', "type": "smallint"},
    # # { 'name': 'subsec_time_digitized', "type": "smallint"},
    # { 'name': 'focal_plane_y_resolution', 'type': 'int_array'},
    # # { 'name': 'focal_plane_resolution_unit', "type": "smallint"},
    # { 'name': 'sensing_method', "type": "smallint"},
    # { 'name': 'interoperability_offset', "type": "smallint"},
    # { 'name': 'f_number', 'type': 'int_array'},
    # { 'name': 'copyright', 'type': 'text_256'},
    # { 'name': 'exposure_program', "type": "smallint"},
    # { 'name': 'custom_rendered', "type": "smallint"},
    # { 'name': 'iso_speed_ratings', "type": "smallint"},
    # { 'name': 'resolution_unit', "type": "smallint"},
    # { 'name': 'exposure_mode', "type": "smallint"},
    # { 'name': 'white_balance', "type": "smallint"},
    # { 'name': 'body_serial_number', 'type': "text"},
    # { 'name': 'digital_zoom_ratio', 'type': 'int_array' 5472, 5472),
    # { 'name': 'scene_capture_type', "type": "smallint"},
    # { 'name': 'orientation', "type": "smallint"},
    # { 'name': 'offset', "type": "smallint"},
    # { 'name': 'y_cb_cr_positioning', "type": "smallint"},
    # /////////////////
    # OTHER DATA
    # ////////////////
    # ...
]

IMAGE_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "exif", IMAGE_DEFAULTS_DEFAULT_FIELDS_INIT, IMAGE_DEFAULTS_DEFAULT_FIELD_PROPS
)



# /////////////////
# FILE EXTENSIONS
# /////////////////

AUDIO_DEFAULTS_VALID_EXT_MIMETYPE = {
    "au": "audio/basic",
    "snd": "audio/basic",
    "mp2": "audio/mpeg",
    "mp3": "audio/mpeg",
    "m4a": "audio/x-aiff",
    "aif": "audio/x-aiff",
    "aifc": "audio/x-aiff",
    "aiff": "audio/x-aiff",
    "m3u": "audio/x-mpegurl",
    "m3u8": "audio/x-mpegurl",
    "ra": "audio/vnd.rn-realaudio",
    "ram": "audio/vnd.rn-realaudio",
    "ogg": "audio/ogg",
    "oga": "audio/ogg",
    "wav": "audio/vnd.wav",
    "wave": "audio/vnd.wav",
    "opus": "audio/opus",
    "mid": "audio/midi",
    "midi": "audio/midi",
}

# /////////////////
# KEYS
# /////////////////


AUDIO_DEFAULTS_HARMONIC_CODES_TO_KEYS = {
    "1A": "Abm",
    "2A": "Ebm",
    "3A": "Bbm",
    "4A": "Fm",
    "5A": "Cm",
    "6A": "Gm",
    "7A": "Dm",
    "8A": "Am",
    "9A": "Em",
    "10A": "Bm",
    "11A": "F#m",
    "12A": "Dbm",
    "1B": "B",
    "2B": "F#",
    "3B": "Db",
    "4B": "Ab",
    "5B": "Eb",
    "6B": "Bb",
    "7B": "F",
    "8B": "C",
    "9B": "G",
    "10B": "D",
    "11B": "A",
    "12B": "E",
}

AUDIO_DEFAULTS_KEYS_TO_HARMONIC_CODES = {
    value: key for key, value in AUDIO_DEFAULTS_HARMONIC_CODES_TO_KEYS.items()
}
AUDIO_DEFAULTS_VALID_KEYS = list(AUDIO_DEFAULTS_HARMONIC_CODES_TO_KEYS.values())
AUDIO_DEFAULTS_VALID_HARMONIC_CODES = list(AUDIO_DEFAULTS_HARMONIC_CODES_TO_KEYS.keys())


# lookup of various key formats to Simplified
AUDIO_DEFAULTS_KEY_LOOKUP = {
    "AMAJOR": "A",
    "AMINOR": "Am",
    "A#MAJOR": "Bb",
    "A#MINOR": "Bbm",
    "BBMAJOR": "Bb",
    "BBMINOR": "Bbm",
    "BMAJOR": "B",
    "BMINOR": "Bm",
    "CMAJOR": "C",
    "CMINOR": "Cm",
    "C#MAJOR": "Db",
    "C#MINOR": "Dbm",
    "DBMAJOR": "Db",
    "DBMINOR": "Dbm",
    "DMAJOR": "D",
    "DMINOR": "Dm",
    "D#MAJOR": "Eb",
    "D#MINOR": "Ebm",
    "EBMAJOR": "Eb",
    "EBMINOR": "Ebm",
    "EMAJOR": "E",
    "EMINOR": "Em",
    "FMAJOR": "F",
    "FMINOR": "Fm",
    "F#MAJOR": "F#",
    "F#MINOR": "F#m",
    "GBMAJOR": "F#",
    "GBMINOR": "F#m",
    "GMAJOR": "G",
    "GMINOR": "Gm",
    "G#MAJOR": "Ab",
    "G#MINOR": "Abm",
    "ABMAJOR": "Ab",
    "ABMINOR": "Abm",
    ###
    "AMAJ": "A",
    "AMIN": "Am",
    "A#MAJ": "Bb",
    "A#MIN": "Bbm",
    "BBMAJ": "Bb",
    "BBMIN": "Bbm",
    "BMAJ": "B",
    "BMIN": "Bm",
    "CMAJ": "C",
    "CMIN": "Cm",
    "C#MAJ": "Db",
    "C#MIN": "Dbm",
    "DBMAJ": "Db",
    "DBMIN": "Dbm",
    "DMAJ": "D",
    "DMIN": "Dm",
    "D#MAJ": "Eb",
    "D#MIN": "Ebm",
    "EBMAJ": "Eb",
    "EBMIN": "Ebm",
    "EMAJ": "E",
    "EMIN": "Em",
    "FMAJ": "F",
    "FMIN": "Fm",
    "F#MAJ": "F#",
    "F#MIN": "F#m",
    "GBMAJ": "F#",
    "GBMIN": "F#m",
    "GMAJ": "G",
    "GMIN": "Gm",
    "G#MAJ": "Ab",
    "G#MIN": "Abm",
    "ABMAJ": "Ab",
    "ABMIN": "Abm",
    ###
    "A": "A",
    "AM": "Am",
    "A#": "Bb",
    "A#M": "Bbm",
    "BB": "Bb",
    "BBM": "Bbm",
    "B": "B",
    "BM": "Bm",
    "C": "C",
    "CM": "Cm",
    "C#": "Db",
    "C#M": "Dbm",
    "DB": "Db",
    "DBM": "Dbm",
    "D": "D",
    "DM": "Dm",
    "EB": "Eb",
    "EBM": "Ebm",
    "EB": "Eb",
    "EBM": "Ebm",
    "E": "E",
    "EM": "Em",
    "F": "F",
    "FM": "Fm",
    "F#": "F#",
    "F#M": "F#m",
    "GB": "F#",
    "GBM": "F#m",
    "G": "G",
    "GM": "Gm",
    "G#": "Ab",
    "G#M": "Abm",
    "AB": "Ab",
    "ABM": "Abm",
    "AB": "Ab",
    "ABM": "Abm",
}

# /////////////////
# BPM
# ////////////


AUDIO_DEFAULTS_VALID_BPM_MIN = 20.0
AUDIO_DEFAULTS_VALID_BPM_MAX = 300.0
AUDIO_DEFAULTS_BPM_DECIMALS = 2

# automatically classify all audio files under this duration as `hits`
AUDIO_DEFAULTS_HIT_IS_DURATION_UNDER_N_SECONDS = 5.0



# local directory (where we store files in local mode)
FILE_STORE_LOCAL_DIR = "~/.dada"
FILE_STORE_LOCAL_AUDITS = False  # whether or not to audit records in local mode
FILE_STORE_LOCAL_VERSIONS = False  # whether or not to version files in local mode

#
FILE_STORE_PUBLIC_ENDPOINT = "https://dist.gl"



# /////////////////
# Document FILE EXTENSIONS
# /////////////////

DOC_DEFAULTS_VALID_EXT_MIMETYPE = {
    "pdf": "application/pdf",
    "txt": "text/plain",
    "text": "text/plain",
    "md": "text/markdown; charset=UTF-8",
    "markdown": "text/markdown; charset=UTF-8",
    "ppt": "application/vnd.ms-powerpoint",
    "pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    "doc": "application/msword",
    "docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "rtf": "application/rtf",
}


# TODO: IMPLEMENT THIS LATER



import os
from dotenv import load_dotenv

load_dotenv()

# app

SECRET_KEY = os.getenv("DADA_SUPER_USER_SECRET_KEY", "dada123")
PORT = os.getenv("DADA_PORT", 3030)
HOST = os.getenv("DADA_HOST", "0.0.0.0")
HTTPS = os.getenv("DADA_HTTPS", "false").lower() == "true"
DEFAULT_BASE_URL = "http://localhost:3030"
if HTTPS:
    DEFAULT_BASE_URL = DEFAULT_BASE_URL.replace("http", "https")
BASE_URL = os.getenv("DADA_BASE_URL", DEFAULT_BASE_URL)
API_VERSION = os.getenv("DADA_API_VERSION", "1.0.0")
API_KEY_HEADER = os.getenv("DADA_API_KEY_HEADER", "X-DADA-API-KEY")
ENV_VAR_PREFIX = os.getenv("DADA_ENV_VAR_PREFIX", "dada")

# db
SQLALCHEMY_DATABASE_URI = os.getenv(
    "DADA_SQLALCHEMY_DATABASE_URI", "postgresql://localhost:5432/dada"
)
SQLALCHEMY_TRACK_MODIFICATIONS = False

# super user
SUPER_USER_NAME = os.getenv("DADA_SUPER_USER_NAME", "gltd")
SUPER_USER_EMAIL = os.getenv("DADA_SUPER_USER_EMAIL", "dev@globally.ltd")
SUPER_USER_PASSWORD = os.getenv("DADA_SUPER_USER_PASSWORD", "dev")
SUPER_USER_API_KEY = os.getenv("DADA_SUPER_USER_API_KEY", "dev")

# filesearch
FILE_SEARCH_RESULTS_PER_PAGE = os.getenv("DADA_FILE_SEARCH_RESULTS_PER_PAGE", 10)

# celery
CELERY_BROKER_URL = os.getenv("DADA_CELERY_BROKER_URL", "redis://localhost:6379")
CELERY_RESULT_BACKEND = os.getenv(
    "DADA_CELERY_RESULT_BACKEND", "redis://localhost:6379"
)
CELERY_ACCEPT_CONTENT = [os.getenv("DADA_CELERY_ACCEPT_CONTENT", "application/json")]
CELERY_RESULT_SERIALIZER = os.getenv("DADA_CELERY_RESULT_SERIALIZER", "json")
CELERY_TASK_SERIALIZER = os.getenv("DADA_CELERY_TASK_SERIALIZER", "json")

# redis cache
REDIS_CACHE_URL = os.getenv("DADA_REDIS_CACHE_URL", "redis://localhost:6379/")
REDIS_CACHE_DB = os.getenv("DADA_REDIS_CACHE_DB", "0")
REDIS_CACHE_KWARGS = os.getenv("DADA_REDIS_CACHE_KWARGS", {})
REDIS_CACHE_SERIALIZER = "pickle"  # TODO: implement something else?

# email
EMAIL_HOST = os.getenv("DADA_EMAIL_HOST", "mail.gandi.net")
EMAIL_FROM_USER = os.getenv("DADA_EMAIL_FROM_USER", "lola@gltd.cat")
EMAIL_FROM_USER_FULL_NAME = os.getenv("DADA_EMAIL_FROM_USER_FULL_NAME", "lola")
EMAIL_FROM_USER_SMTP_PORT = int(os.getenv("DADA_EMAIL_SMTP_PORT", 587))
EMAIL_FROM_USER_IMAP_PORT = int(os.getenv("DADA_EMAIL_IMAP_PORT", 998))
EMAIL_PASSWORD = os.getenv("DADA_EMAIL_PASSWORD")

# filestore
TEMPFILE_DIR = os.getenv("DADA_TMP_DIR", "/tmp")  # override for docker environment
LOCAL_DIR = os.getenv("DADA_LOCAL_DIR", os.path.expanduser("~/.dada/stor/"))
S3_PLATFORM = os.getenv("DADA_S3_PLATFORM", "do")
if S3_PLATFORM == "do":
    S3_HOST = "digitaloceanspaces.com"
else:
    S3_HOST = "s3.amazonaws.com"  # TODO: check this.

S3_BUCKET = os.getenv("DADA_S3_BUCKET", "dada.globally.ltd")
S3_ACCESS_KEY_ID = os.getenv("DADA_S3_ACCESS_KEY_ID")
S3_SECRET_ACCESS_KEY = os.getenv("DADA_S3_SECRET_ACCESS_KEY")
S3_REGION_NAME = os.getenv("DADA_S3_REGION_NAME", "nyc3")
S3_BASE_URL = os.getenv(
    "DADA_S3_BASE_URL", f"https://{S3_BUCKET}.{S3_REGION_NAME}.{S3_HOST}"
)
S3_ENDPOINT_URL = os.getenv(
    "DADA_S3_ENDPOINT_URL", f"https://{S3_REGION_NAME}.{S3_HOST}"
)

# api spec
APISPEC_SWAGGER_URL = os.getenv("DADA_APISPEC_SWAGGER_URL", "/spec/")
APISPEC_SWAGGER_UI_URL = os.getenv("DADA_APISPEC_SWAGGER_UI_URL", "/docs/")

# logging settings
LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {
        "default": {"format": ("<%(levelname)s> %(message)s [%(asctime)s]")}
    },
    "datefmt": "%I:%M:%S",
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "level": "INFO",
            "formatter": "default",
        }
    },
    "root": {"level": "INFO", "handlers": ["console"]},
}

LOG_LEVEL = os.getenv("DADA_LOG_LEVEL", "debug")



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT CODE FIELDS
# ////////////////

CODE_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["code"],
}

CODE_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "language",
        "type": "text",
        "info": "The inferred programming language",
    },
]

CODE_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "code", CODE_DEFAULTS_DEFAULT_FIELDS_INIT, CODE_DEFAULTS_DEFAULT_FIELD_PROPS
)



import os

DEBUG = False
SQLALCHEMY_DATABASE_URI = os.getenv(
    "DADA_SQLALCHEMY_DATABASE_URI_PROD", "postgresql://localhost:5432/dada"
)
# SQLALCHEMY_ECHO = True
S3_BUCKET = os.getenv("DADA_S3_BUCKET_PROD", "dada")



MODEL_DEFAULTS_VALID_EXT_MIMETYPE = {
    "gltf": "model/gltf",
    "glb": "model/gltf",
    "glk": "model/glk",
    "stl": "application/octect-stream",
    "obj": "application/octect-stream",
    "fbx": "application/octect-stream",
}



# ///////////////////////////////////////
# Random Generation
# ///////////////////////////////////////


FOLDER_DEFAULTS_RANDOM_FOLDER_NAME_WORD_2 = [
    "chillers",
    "smokers",
    "freakers",
    "funkers",
    "hackers",
    "tweakers",
    "greasers",
    "seizuers",
    "leisuers",
    "vibers",
    "pleasers",
    "melters",
    "healers",
    "feelers",
    "breakers",
    "fillers",
    "clearers",
    "tools",
]



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT DOC FIELDS
# ////////////////

DOC_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["doc"],
}

DOC_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "text_contents",
        "type": "text",
        "info": "The document's extracted text contents",
        "is_searchable": True,
    },
]

DOC_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "doc", DOC_DEFAULTS_DEFAULT_FIELDS_INIT, DOC_DEFAULTS_DEFAULT_FIELD_PROPS
)



import os

# ///////////////////////////////////////
# Env Configurations
# ///////////////////////////////////////


FIELD_DEFAULTS_FIELD_TABLE_SCHEMA = os.getenv(
    "DADA_FIELD_DEFAULTS_FIELD_TABLE_SCHEMA", "fields"
)

FIELD_DEFAULTS_FIELD_TABLE_PREFIX = os.getenv(
    "DADA_FIELD_DEFAULTS_FIELD_TABLE_SCHEMA", "file_field"
)

# ///////////////////////////////////////
# Search Configuration
# ///////////////////////////////////////

FIELD_DEFAULTS_SEARCHABLE_FIELD_SUFFIX = "vector"

# ///////////////////////////////////////
# FIELD ACCEPTS ENTITY TYPES
# ///////////////////////////////////////

FIELD_DEFAULTS_ENTITY_TYPES = [
    "file",
    "folder",
    "file_folder",
    "folder_desktop",
    "desktop",
    "tag",
    "user",
]



# /////////////////
# Code FILE EXTENSIONS
# /////////////////

CODE_DEFAULTS_VALID_EXT_MIMETYPE = {
    "py": "text/x-python",
    "js": "text/javascript",
    "css": "text/css",
    "html": "application/html",
    "xhtml": "application/xhtml+xml",
    "j2": "application/x-jinja",
    "r": "application/x-rstats",
    "sh": "application/x-sh",
    "zsh": "application/x-zsh",
}



import os
import dotenv

# ENV-BASED CREDENTIALS
# load .env file
ENV_FILE = os.getenv("DATA_ENV_FILE", os.path.expanduser("~/.dada/.env"))
if not ENV_FILE or not os.path.exists(ENV_FILE):
    dotenv.load_dotenv()
else:
    dotenv.load_dotenv(ENV_FILE)

ENV = os.getenv("DADA_ENV", "dev")

if not ENV:
    raise RuntimeError("No DADA_ENV supplied.")

# load constants
from dada_settings.core import *
from dada_settings.cons import *

# override with environments
if ENV == "prod":
    from dada_settings.prod import *

if ENV == "dev":
    from dada_settings.dev import *

if ENV == "test":
    from dada_settings.test import *

if ENV == "docker":
    from dada_settings.docker import *



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT DATA FIELDS
# ////////////////

DATA_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["data"],
}

DATA_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "n_rows",
        "type": "int",
        "info": "The number of rows in the dataset",
    },
    {
        "name": "n_cols",
        "type": "int",
        "info": "The number of columns in the dataset",
    },
    {
        "name": "dada_schema",
        "type": "json",
        "info": "The inferred dada schema",
    },
]

DATA_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "data", DATA_DEFAULTS_DEFAULT_FIELDS_INIT, DATA_DEFAULTS_DEFAULT_FIELD_PROPS
)



from dada_settings.utils import join_fields

# ///////////////////////////////////////
# Default TAG Generation
# ///////////////////////////////////////

# DEFAULT TAGS
TAG_DEFAULTS = [
    {"name": "globally-ltd", "info": "content related to globally.ltd"},
]


# /////////////////
# DEFAULT TAG FIELDS
# ////////////////

TAG_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["tag"],
    "accepts_file_types": ["all"],
}

TAG_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "assoicated_url",
        "type": "url",
        "info": "A url associated with a website",
    },
]

TAG_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "tag", TAG_DEFAULTS_DEFAULT_FIELDS_INIT, TAG_DEFAULTS_DEFAULT_FIELD_PROPS
)



# ///////////////////////////////////////
# RANDOM GENERATION
# ///////////////////////////////////////


FIELD_DEFAULTS_RANDOM_GENRE_WORD_1 = [
    "jazz",
    "fizz",
    "hip",
    "yacht",
    "future",
    "cat",
    "dog",
    "ambient" "brunch",
    "twee",
    "future",
    "techno",
    "psych",
    "hypno",
    "anarcho",
    "marxist",
    "wavey",
    "vibey",
    "leftfield",
    "outsider",
    "alt",
    "alternative",
    "eclectic",
    "business",
    "serious",
    "contemporary",
]

FIELD_DEFAULTS_RANDOM_GENRE_WORD_2 = [
    "house",
    "disco",
    "hop",
    "trip",
    "bass",
    "ambient",
    "rock",
    "fusion",
    "funk",
    "dm",
    "music",
    "vibe",
    "wave",
    "drill",
    "trap",
    "boogie",
    "tech",
    "matic",
    "step",
]

FIELD_DEFAULTS_RANDOM_ARTIST_NAME_WORD_1 = [
    "dj",
    "mc",
    "kool dj",
    "ancient",
    "samurai",
    "her majesty",
    "The",
    "fresh",
    "kool mo",
    "jazzy",
    "amadeus",
    "karl",
    "jon",
    "sally",
    "olga",
    "bell",
    "toni",
    "M",
    "J",
    "LL",
    "BB",
    "CC",
    "ZZ",
    "No",
    "Yes",
]


FIELD_DEFAULTS_RANDOM_ALBUM_NAME_WORD_1 = [
    "The",
    "A",
    "One",
    "My",
    "Your",
    "His",
    "Her",
    "Their",
    "Our",
    "It's",
]

FIELD_DEFAULTS_RANDOM_ALBUM_NAME_WORD_2 = [
    "city",
    "county",
    "village",
    "town",
    "place",
    "mountain",
    "club",
    "commune",
    "squire",
    "chapel",
    "dancefloor",
    "disco",
    "valley",
    "isthmus",
    "port",
]
FIELD_DEFAULTS_RANDOM_ALBUM_NAME_WORD_3 = [
    "called",
    "for",
    "against",
    "enraged with",
    "named",
    "known as",
    "they called",
    "she called",
    "he called",
    "that unified in",
    "that fell apart because of",
    "enshrouded in",
    "coverer in",
    "awash in",
]
FIELD_DEFAULTS_RANDOM_ALBUM_NAME_WORD_4 = [
    "justice",
    "loathing",
    "solitude",
    "groove",
    "funk",
    "revolution",
    "retribution",
    "disobedience",
    "anger",
    "pride",
    "lust",
    "envy",
    "sin",
    "greed",
    "love",
    "emotions",
    "feelings",
    "tempers",
]
FIELD_DEFAULTS_RANDOM_ALBUM_NAME_WORD_5 = [
    "EP",
    "LP",
    "12'",
    "7'",
    "2x LP",
    "3x LP",
    "10x LP",
    "Reissue",
    "Greatest Hits",
]

# FIELD_DEFAULTS_RANDOM_TEXT_PATH = path.get_env(
#     "DADA_FIELD_DEFAULTS_RANDOM_TEXT_PATH",
#     path.here(__file__, "assets/raw/txt/mobydick.txt"),
# )
FIELD_DEFAULTS_RANDOM_TEXT = (
    "foo bar baz uqbar dada  dsjafhkasdjhflaskj.dhfakl wuejfilauekjsbfdkl uajsbfklajebsrflk auabejwslkf jbaeslkedfdfjsb alkweejkasfdb lkasejdhbbdfslk uwaesvbdflkuajehsbdf lkjasbdhv fklasfmas "
    * 100
)
# with open(FIELD_DEFAULTS_RANDOM_TEXT_PATH, "r") as f:
#     FIELD_DEFAULTS_RANDOM_TEXT += text.rm_whitespace(f.read())
FIELD_DEFAULTS_RANDOM_TEXT_LENGTH = len(FIELD_DEFAULTS_RANDOM_TEXT)



TAG_DEFAULTS_ACCEPTS_ENTITY_TYPES = [
    "file",
    "field",
    "folder",
    "desktop",
    "user",
    "theme",
]

TAG_DEFAULTS_ACCEPTS_ENTITY_TYPES_ALL = TAG_DEFAULTS_ACCEPTS_ENTITY_TYPES + ["all"]
TAG_DEFAULTS_ACCEPTS_ENTITY_TYPES_DEFAULT = ["all"]



from dada_settings.utils import join_fields

BUNDLE_DEFAULTS = []


# /////////////////
# DEFAULT BUNDLE FIELDS
# ////////////

BUNDLE_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["bundle"],
    "accepts_file_subtypes": ["all"],
}

BUNDLE_DEFAULTS_DEFAULT_FIELDS_INIT = [{"name": "contents", "type": "text_array"}]

BUNDLE_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "bundle", BUNDLE_DEFAULTS_DEFAULT_FIELDS_INIT, BUNDLE_DEFAULTS_DEFAULT_FIELD_PROPS
)



import os

# ///////////////////////////////////////
# Env Configurations
# ///////////////////////////////////////


FILE_DEFAULTS_NUMBER_RANDOM_FILES = os.getenv(
    "DADA_FILE_DEFAULTS_NUMBER_RANDOM_FILES", 200
)

FILE_DEFAULTS_NUMBER_DEFAULT_FILES = os.getenv(
    "DADA_FILE_DEFAULTS_NUMBER_DEFAULT_FILES", 20
)

FILE_DEFAULTS_NUMBER_DEFAULT_FIELDS_PER_FILE = os.getenv(
    "DADA_FILE_DEFAULTS_NUMBER_DEFAULT_FIELDS_PER_FILE", 10
)

# ///////////////////////////////////////
# Default Files
# ///////////////////////////////////////


FILE_DEFAULTS_FILE_FIXTURE_DIR = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "assets")
)
FILE_DEFAULTS_FILE_FIXTURE_1 = os.path.join(
    FILE_DEFAULTS_FILE_FIXTURE_DIR, "audio/tracks/space-time-motion.mp3"
)
FILE_DEFAULTS_FILE_FIXTURE_2 = os.path.join(
    FILE_DEFAULTS_FILE_FIXTURE_DIR, "assets/audio/tracks/eril-brinh2.mp3"
)

FILE_DEFAULTS_FILE_FIXTURES = [
    FILE_DEFAULTS_FILE_FIXTURE_1,
    FILE_DEFAULTS_FILE_FIXTURE_2,
]

# ///////////////////////////////////////
# File type schema
# ///////////////////////////////////////


FILE_DEFAULTS_FILE_TYPES = [
    "audio",
    "bundle",
    "video",
    "image",
    "model",
    "data",
    "doc",
    "code",
    "raw",
]
FILE_DEFAULTS_FILE_TYPES_ALL = FILE_DEFAULTS_FILE_TYPES + ["all"]
FILE_DEFAULTS_DEFAULT_TYPE = "raw"
FILE_DEFAULTS_FILE_SUBTYPE_SCHEMA = {
    "audio": ["hit", "loop", "track", "mix", "raw", "midi"],
    "bundle": [
        "gltf",
        "ableton_project",
        "logic_project",
        "git_repo",
        "audio_album",
        "rekordbox_library",
        "itunes_library",
        "site",
        "app",
        "raw",
    ],
    "video": ["loop", "clip", "raw"],  # TODO add more here
    "image": ["loop", "static", "raw"],  # TODO Add more here
    "model": ["3d"],  # TODO Add more here
    "data": ["raw"],
    "code": ["raw"],
    "raw": ["raw"],
}

FILE_DEFAULTS_FILE_SUBTYPES = list(
    set(
        [val for values in FILE_DEFAULTS_FILE_SUBTYPE_SCHEMA.values() for val in values]
    )
)
FILE_DEFAULTS_FILE_SUBTYPES_ALL = FILE_DEFAULTS_FILE_SUBTYPES + ["all"]
FILE_DEFAULTS_DEFAULT_SUBTYPE = "raw"
FILE_DEFAULTS_DEFAULT_FILE_SUBTYPE_FOR_FILE_TYPE = {
    "audio": "track",
    "bundle": "raw",
    "data": "raw",
    "video": "clip",
    "image": "static",
    "model": "3d",
    "doc": "raw",
    "code": "raw",
    "raw": "raw",
}

FILE_DEFAULTS_DEFAULT_FILE_TYPE = "raw"
FILE_DEFAULTS_DEFAULT_FILE_SUBTYPE = "raw"
FILE_DEFAULTS_DEFAULT_EXT = ""
FILE_DEFAULTS_DEFAULT_MIMETYPE = "application/octet-stream"



import random

from dada_settings.utils import join_fields

from dada_settings.models.desktop_defaults import (
    DESKTOP_DEFAULTS_NUMBER_DEFAULT_DESKTOPS,
)

# ///////////////////////////////////////
# Default Folder Generation
# ///////////////////////////////////////


def gen_desktop():
    """"""
    return [random.choice(range(1, DESKTOP_DEFAULTS_NUMBER_DEFAULT_DESKTOPS + 1, 1))]


FOLDER_DEFAULTS = [
    {
        "name": "to_dj",
        "emoji": "heavy_division_sign",
        "info": "Your dj inbox",
        "is_private": True,
        "desktop_id": gen_desktop(),
    },
    {
        "name": "inbox",
        "emoji": "inbox_tray",
        "info": "Stuff to organize",
        "is_private": True,
        "desktop_id": gen_desktop(),
    },
    {
        "name": "to_share",
        "emoji": "link",
        "info": "Stuff to share",
        "is_private": False,
        "desktop_id": gen_desktop(),
    },
]

FOLDER_DEFAULTS_NAMES = [f["name"] for f in FOLDER_DEFAULTS]
FOLDER_DEFAULTS_NUMBER_DEFAULT_FOLDERS = len(FOLDER_DEFAULTS_NAMES)


FOLDER_DEFAULTS_DEFAULT_FIELDS = []



IMAGE_DEFAULTS_VALID_EXT_MIMETYPE = {
    "jpg": "image/jpeg",
    "jpeg": "image/jpeg",
    "png": "image/png",
    "bmp": "image/bmp",
    "gif": "image/gif",
    "tif": "image/tiff",
    "tiff": "image/tiff",
    "eps": "application/postscript",
    "svg": "image/svg+xml",
    "ico": "image/vnd.microsoft.icon",
    "webp": "image/webp",
}

# default image metadata extraction
IMAGE_DEFAULTS_DEFAULT_METADATA_EXTRACTION = "exif"



import os

from dada_settings.utils import join_fields


# super user
SUPER_USER_NAME = os.getenv("DADA_SUPER_USER_NAME", "gltd")
SUPER_USER_EMAIL = os.getenv("DADA_SUPER_USER_EMAIL", "dev@globally.ltd")
SUPER_USER_PASSWORD = os.getenv("DADA_SUPER_USER_PASSWORD", "dada")
SUPER_USER_API_KEY = os.getenv("DADA_SUPER_USER_API_KEY", "dev")

# default users
USER_DEFAULTS = [
    {
        "name": SUPER_USER_NAME,
        "email": SUPER_USER_EMAIL,
        "password": SUPER_USER_PASSWORD,
        "api_key": SUPER_USER_API_KEY,
        "fields": {"settings": {"is_default": True}},
    }
]

# /////////////////
# DEFAULT USER FIELDS
# ////////////////

USER_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["user"],
}

USER_DEFAULTS_DEFAULT_FIELDS_INIT = [
    {
        "name": "settings",
        "type": "json",
        "info": "Arbitrary user settings",
    },
]

USER_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "user", USER_DEFAULTS_DEFAULT_FIELDS_INIT, USER_DEFAULTS_DEFAULT_FIELD_PROPS
)



import os

SQLALCHEMY_DATABASE_URI = os.getenv(
    "DADA_SQLALCHEMY_DATABASE_URI_TEST", "postgresql://localhost:5432/dada_test"
)
CELERY_ALWAYS_EAGER = True
S3_BUCKET = os.getenv("DADA_S3_BUCKET_TEST", "test.dada.globally.ltd")



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT VIDEO FIELDS
# ////////////////

VIDEO_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["video"],
}

VIDEO_DEFAULTS_DEFAULT_FIELDS_INIT = [
    # toplevel information
    {"name": "video_has_audio", "type": "bool"},
    {"name": "probe_score", "type": "int"},
    {"name": "format_name", "type": "text", "is_searchable": True},
    {"name": "created_date", "type": "date"},
    # video-specific attributes
    {"name": "video_codec_name", "type": "text"},
    {"name": "video_color_range", "type": "text"},
    {"name": "video_width", "type": "smallint"},
    {"name": "video_height", "type": "smallint"},
    {"name": "video_level", "type": "int"},
    {"name": "video_bit_rate", "type": "int"},
    {"name": "video_duration", "type": "num"},
    {"name": "video_start_time", "type": "num"},
    {"name": "video_nb_frames", "type": "int"},
    {"name": "sample_aspect_ratio_x", "type": "smallint"},
    {"name": "sample_aspect_ratio_y", "type": "smallint"},
    {"name": "display_aspect_ratio_x", "type": "smallint"},
    {"name": "display_aspect_ratio_y", "type": "smallint"},
    {"name": "avg_frame_rate", "type": "num"},
    # audio-specific attributes
    {"name": "audio_codec_name", "type": "text"},
    {"name": "audio_is_stereo", "type": "bool"},
    {"name": "audio_sample_rate", "type": "int"},
    {"name": "audio_bit_rate", "type": "int"},
    {"name": "audio_nb_frames", "type": "int"},
    {"name": "audio_language", "type": "text"},
    # audio dipositions
    {"name": "audio_disposition_is_default", "type": "bool"},
    {"name": "audio_disposition_is_dub", "type": "bool"},
    {"name": "audio_disposition_is_original", "type": "bool"},
    {"name": "audio_disposition_is_comment", "type": "bool"},
    {"name": "audio_disposition_is_lyrics", "type": "bool"},
    {"name": "audio_disposition_is_karaoke", "type": "bool"},
    {"name": "audio_disposition_is_forced", "type": "bool"},
    {"name": "audio_disposition_is_hearing_impaired", "type": "bool"},
    {"name": "audio_disposition_is_visual_impaired", "type": "bool"},
    {"name": "audio_disposition_is_clean_effects", "type": "bool"},
    {"name": "audio_disposition_is_attached_pic", "type": "bool"},
    {"name": "audio_disposition_is_timed_thumbnails", "type": "bool"},
]

VIDEO_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "ffp", VIDEO_DEFAULTS_DEFAULT_FIELDS_INIT, VIDEO_DEFAULTS_DEFAULT_FIELD_PROPS
)



from dada_settings.utils import join_fields

# /////////////////
# DEFAULT AUDIO FIELDS
# ////////////
AUDIO_DEFAULTS_DEFAULT_FIELD_PROPS = {
    "accepts_entity_types": ["file"],
    "accepts_file_types": ["audio"],
}

# TODO: Move this to an EXTRACT TASK!
AUDIO_DEFAULTS_DEFAULT_FIELDS_INIT = [
    # /////////////////
    # ID3 TAGS
    # ////////////
    # SEARCHABLE
    {"name": "artist_name", "type": "text", "is_searchable": True},
    {"name": "original_artist_name", "type": "text"},
    {"name": "album_artist_name", "type": "text"},
    {"name": "track_title", "type": "text", "is_searchable": True},
    {"name": "album_name", "type": "text", "is_searchable": True},
    {"name": "original_album_name", "type": "text"},
    {"name": "genre", "type": "text"},
    {"name": "label_name", "type": "text"},
    {"name": "comment", "type": "text"},
    {"name": "encoded_by", "type": "text"},
    # Integers
    {"name": "bit_rate", "type": "int"},
    {"name": "sample_rate", "type": "int"},
    {"name": "track_num", "type": "int"},
    {"name": "track_total", "type": "int"},
    {"name": "disc_num", "type": "int"},
    {"name": "disc_total", "type": "int"},
    {"name": "bytes", "type": "int"},
    {"name": "year", "type": "int"},
    {"name": "compilation", "type": "int"},
    {"name": "num_measures", "type": "int"},
    {"name": "num_bars", "type": "int"},
    # Dates
    {"name": "release_date", "type": "date_tz"},
    {"name": "original_date", "type": "date_tz"},
    # URLs (not type enforced)
    {"name": "webpage", "type": "text"},
    {"name": "publisher_webpage", "type": "text"},
    {"name": "url", "type": "text"},
    # Numeric
    {"name": "bpm", "type": "num"},
    {"name": "duration", "type": "num"},
    # Boolean
    {"name": "is_stereo", "type": "bool"},
    # ENUM
    {
        "name": "musical_key",
        "type": "text",
        # "options": AUDIO_DEFAULTS_VALID_KEYS, # TODO: Figure out this bug
    },
    {
        "name": "harmonic_code",
        "type": "text",
        # "options": AUDIO_DEFAULTS_VALID_HARMONIC_CODES, # TODO: Figure out this bug
    },
]

AUDIO_DEFAULTS_DEFAULT_FIELDS = join_fields(
    "id3", AUDIO_DEFAULTS_DEFAULT_FIELDS_INIT, AUDIO_DEFAULTS_DEFAULT_FIELD_PROPS
)
